#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2005 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#	DrPython is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
#Functions for getting information about blocks

import re

reNotEmpty = re.compile('\S')

def CheckForComment(line):
	line = line.strip()
	if len(line) > 0:
		return line[0] == '#'
	return False

def CheckForExplicitLineJoining(lines, current):
	if current > 0:
		line = lines[current - 1].strip()
		l = len(line)
		if l > 0:
			return line[l - 1] == '\\'
				
	return False

def CheckIfBlockStart(lines, targetline):
	LineIndent = GetLineIndentation(lines, targetline)
	
	if LineIndent == -1:
		LineIndent = FindValidLine(lines, targetline)
		if LineIndent == -1:
			return False
	
	#try till the end
	l = len(lines)
	li = 1
	result = -1
	while result == -1:
		if targetline + li >= l:
			break
		result = GetLineIndentation(lines, targetline+li)
		li += 1
		
	if result > LineIndent:
		return li - 1
	return False

def FindValidLine(lines, targetline):
	#try till the end
	l = len(lines)
	li = 1
	result = -1
	while result == -1:
		if targetline + li >= l:
			break
		if CheckForComment(lines[targetline+li]):
			result = -1
		else:
			result = GetLineIndentation(lines, targetline+li)
		if result == -1:
			if targetline - li <= 0:
				break
			if CheckForComment(lines[targetline-li]):
				result = -1
			else:
				result = GetLineIndentation(lines, targetline-li)
		li += 1
			
	return result
	
def GetBlockStart(lines, targetline):
	LineIndent = GetLineIndentation(lines, targetline)
	
	if LineIndent == -1:
		LineIndent = FindValidLine(lines, targetline)
		if LineIndent == -1:
			return -1

	x = targetline - 1
	while x > 0:
		i = GetLineIndentation(lines, x)
		if i != -1:
			if i < LineIndent:
				return x
		x -= 1
			
	return 0

def GetBlockEnd(lines, targetline):
	l = len(lines)
	LineIndent = GetLineIndentation(lines, targetline)
		
	if LineIndent == -1:
		LineIndent = FindValidLine(lines, targetline)
		if LineIndent == -1:
			return -1

	x = targetline + 1
	last = targetline
	while x < l:
		i = GetLineIndentation(lines, x)
		if i != -1:
			if i >= LineIndent:
				last = x
			if i < LineIndent:
				return last
		x += 1
		
	return len(lines) - 1

def GetKeyWordStart(lines, targetline, kword, kwordlength):
	start = GetBlockStart(lines, targetline)
	if kwordlength == 0:
		return start
	while lines[start].strip()[:kwordlength] != kword:
		start = GetBlockStart(lines, start)
		if start == 0:
			break
	return start

def GetKeyWordEnd(lines, targetline, kword, kwordlength):
	if (kwordlength == 0) or (lines[targetline].strip()[:kwordlength] == kword):
		startofblock = CheckIfBlockStart(lines, targetline)
		if startofblock:
			targetline += startofblock
	else:
		targetline = GetKeyWordStart(lines, targetline, kword, kwordlength) + 1
		
	end = GetBlockEnd(lines, targetline)
	
	return end

def GetLineIndentation(lines, current):
	line = lines[current]
	#Check For Comments
	if CheckForComment(line):
		return -1
	
	#Check if this line is continuing over from the previous
	if CheckForExplicitLineJoining(lines, current):
		return -1
	
	#Get on with it!
	result = reNotEmpty.search(line)
	if result is None:
		return -1
	return result.start()

def GetLines(Document):
	text = Document.GetText()
	eol = Document.GetEndOfLineCharacter()
	text = RemoveComments(text, eol)
	return text.split(eol)

def GoToBlockEnd(Document, kword=''):
	lines = GetLines(Document)
	kwordl = len(kword)
	l = GetKeyWordEnd(lines, Document.GetCurrentLine(), kword, kwordl)
	i = GetLineIndentation(lines, l)
	if i < 0:
		i = 0
	Document.GotoLine(l)
	p = Document.PositionFromLine(l) + i
	Document.GotoPos(p)

def GoToBlockStart(Document, kword=''):
	lines = GetLines(Document)
	kwordl = len(kword)
	l = GetKeyWordStart(lines, Document.GetCurrentLine(), kword, kwordl)
	i = GetLineIndentation(lines, l)
	if i < 0:
		i = 0
	Document.GotoLine(l)
	p = Document.PositionFromLine(l) + i
	Document.GotoPos(p)

def RemoveComments(text, eol):
	text = removeStringTripleQuotedWith(text, "'''")
	text = removeStringTripleQuotedWith(text, '"""')
		
	return text	
	
def removeStringTripleQuotedWith(text, target):
	start = text.find(target)
	while start > -1:
		end = text[start+3:].find(target)
		if end == -1:
			text = text[:start]
		else:
			end = start + 3 + end
			text = text[:start] + spacefill((end - start) + 3) + text[end+3:]
		start = text.find(target)
	return text

def spacefill(length):
	result = ''
	for x in range(length):
		result += ' '
	return result